;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x12f0891715b8cc36b3220124b7d92868 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xdbe3115dd226f69ffb6c9ee30e6a47ec . "./code.cg2js/cg-printing.fasl")
(#xdd4fcca8a5aa43d7a5b5280cba795253 . "./code.cg2js/cg-caret.fasl")
(#x558996f473d9bd27930a65c5db652171 . "./code.cg2js/cg-item-list.fasl")
(#xeccda8acde2d18e1a820fc7ab37c9c2d . "./code.cg2js/cg-progress-indicator.fasl")
(#x6b01e1283d75ea20ab861aa8555a7288 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xfa9dd6f99301b3d19421655913c9f891 . "./code.cg2js/cg-outline.fasl")
(#x8d02d60613ef97663599c6c3b679f191 . "./code.cg2js/cg-palette.fasl")
(#x470c77bf7d790e0ce0c4cce5b17286b1 . "./code.cg2js/cg-selected-object.fasl")
(#x5cbaaa141b1bb72c40b5af527f786e80 . "./code.cg2js/cg-status-bar.fasl")
(#x889b4978a5bb0e39800b20d054f70f98 . "./code.cg2js/cg-wrap-string.fasl")
(#xe3e3e095ed3afe02a72f02240bc84677 . "./code.cg2js/cg-fill-texture.fasl")
(#x4d547923cd3fbdd2bb063b128090636c . "./code.cg2js/cg-layout.fasl")
(#x18df1145c72213d51bdc82570662a679 . "./code.cg2js/cg-timer.fasl")
(#x4caab92d7d3cdb0beb902f924072abed . "./code.cg2js/cg-dialog-item.fasl")
(#xe640c4c458254e1d0ca198fa6b8ee88c . "./code.cg2js/cg-button.fasl")
(#x4c3fe09f5ac21bee42a74cb1d8177e43 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x3df74d51f3981f2403dbc14f7d82ac0a . "./code.cg2js/cg-tray.fasl")
(#x5ddd29cb7336c95041381e2cf14de8c0 . "./code.cg2js/cg-plot-widget.fasl")
(#xd223c5aaba2318a3bf3086abd683a1bf . "./code.cg2js/cg-pixmap.fasl")
(#xb79cd92987afabea7a44469dcb6d0c2f . "./code.cg2js/cg-font-dialog.fasl")
(#xdbc99b9371423dd2c63546a526443031 . "./code.cg2js/cg-os-widget.fasl")
(#xb8d79275adb9abcbc7196ba68dd9f9dd . "./code.cg2js/cg-text-or-combo.fasl")
(#x77e24f3877202f68d60f0f9c36fdd698 . "./code.cg2js/cg-trackbar.fasl")
(#xcba37a5b23ecee9c568d1de672fd278a . "./code.cg2js/cg-up-down-control.fasl")
(#xec2f4c0c40afb4434b2bd5e9f542945a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x8cf898468c5e12b932eb29de2ec22558 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x09eee33b1a3288030b1a8e975e751725 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x9cf738728acbd575ea535a0685926762 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x68324a6ec88db3194f351410a2f028da . "./code.cg2js/cg-chart-widget.fasl")
(#x41e5bbec5522920593240188d773eb1a . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x3a889cb7f0e3c90378aabc90c0405621 . "./code.cg2js/cg-group-box.fasl")
(#xcd6255dc23643d7a90148a709a132b43 . "./code.cg2js/cg-static-text.fasl")
(#xac201cf2df96f493efc50ed43383ce1b . "./code.cg2js/cg-class-slot-grid.fasl")
(#x0867ca4510a88bd01505bcdd12fb03c4 . "./code.cg2js/cg-color-dialog.fasl")
(#x6de8c2185b0a593fe11b90f20c8e7275 . "./code.cg2js/cg-intersect.fasl")
(#x6c821a1ccf8154e83dfd27e2d6ec9b8b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x72c29fff48272416f11bbb289dffa9e4 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xef5cf1d5b62ef21c04fb79932d6bd218 . "./code.cg2js/cg-html-dialog.fasl")
(#x65cd466b0fd4583e3098ef0b7cd149df . "./code.cg2js/cg-lettered-menu.fasl")
(#x72446df7409f09242ad34f4bb63bf750 . "./code.cg2js/cg-class-grid.fasl")
(#xb6876a4a2164b03f7f201b0dabad8e3b . "./code.cg2js/cg-pixmap-printing.fasl")
(#x8eefd8546281bed1f05faf925070b0ae . "./code.cg2js/cg-calendar.fasl")
(#xd25f6ba962e02613151a3e01c850245f . "./code.cg2js/cg-editable-text.fasl")
(#xb53bfe98dfbd383ef1b99309f1f8ec6a . "./code.cg2js/cg-property.fasl")
(#x082365452084b3a2daa1fb2f5ac580b5 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd48a5fa8bfd0ae5c6cef3fef1722302b . "./code.cg2js/cg-grid-widget.fasl")
(#xfabb46105fe828055fdd4b4d916af63a . "./code.cg2js/cg-paren-matching.fasl")
(#x399827456554224738a4a19d601fc50d . "./code.cg2js/cg-dev.fasl")
(#xec400eff166e698a24dfc8757683be32 . "./code.cg2js/cg-drawable.fasl")
(#x277e4ab8aa771cfdf0a8ae60638ca165 . "./code.cg2js/cg-curve.fasl")
(#xf0c552318bf3e82c61f0018656e0467f . "./code.cg2js/cg-lisp-widget.fasl")
(#xa260b1ea041ae4a596d156b126788403 . "./code.cg2js/cg-dde-utils.fasl")
(#x2ff20410974762bfc19acde66b24db8d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xd9aa5f7c40eb62f3f93095de35f18256 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xad1800655a2b363457ae900bab433d5d . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xea3fd8414348b369386fd203849e0ff1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xc4a6ce0308a2b71628acd51a62969e92 . "./code.cg2js/cg-lamp.fasl")
(#x799052f10e4b7fc06d466fe0ff668eb6 . "./code.cg2js/cg-common-control.fasl")
(#xc015b33d0544068840fcd7db41309d0e . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xfb8e3bcdfbc7be646c8aa500ff7e8e6e . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x429e91e11f17db9b77426a23df30d340 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x745daefff5d480112fab6884c1e57d0e . "./code.cg2js/cg-project-window.fasl")
(#xa727fb8ac3067fb6640b753fbdc17aaa . "./code.cg2js/cg-sample-file-menu.fasl")
(#x208d03417eb2bc3f9a19109dd6d0139c . "./code.cg2js/cg-class-support.fasl")
(#x58543ace8be235b69b9b23c84f47659f . "./code.cg2js/cg-multi-picture-button.fasl")
(#xfaf5c7db48a1e41c4df11fcb37e8c208 . "./code.cg2js/cg-yes-no-list.fasl")
(#xaee824af05fd0be6a19f2cb5d998d227 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xb0ac7a131cf57f4d852063f9a902be90 . "./code.cg2js/cg-string-dialog.fasl")
(#x8b7a104a060edd24fd19eb49f451c90b . "./code.cg2js/cg-menu-tooltip.fasl")
(#x4e810e051cf91db19ae08b55f422bf7f . "./code.cg2js/cg-dropping-outline.fasl")
(#xc81625f8a4a0ff8d5fe4f0aeb71f46b7 . "./code.cg2js/cg.fasl")
(#x456cb15dea7f29267fd30e5faf57bd24 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x92bc756db352ad86172a71ac4ee2d625 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3f933b8504c8586a56132feeb13ce16d . "./code.cg2js/cg-yes-no-string.fasl")
(#x03a1dca8dc987e361c2ede0d07a98850 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x395a88b365390d5dc989f4dc88a2a1ce . "./code.cg2js/cg-web-browser.fasl")
(#x6802518bb175adee094570b8a2070ebd . "./code.cg2js/cg-clipboard-stack.fasl")
(#xd2b16f05ce0e05f03f3726a2682941ed . "./code.cg2js/cg-drag-and-drop.fasl")
(#xc1941a93050109606ecbdceb37b94921 . "./code.cg2js/cg-message-dialog.fasl")
(#xa632507e9a914df2cd9a538acc6e8bfd . "./code.cg2js/cg-comtab.fasl")
(#x21056cdf9178ea8d5d2c761f40a8bb0b . "./code.cg2js/cg-base.fasl")
(#xd6391d4cde7c4d2d3540e4e8d82fd32f . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5f539b489cb3641364faecb2d4f6ddef . "./code.cg2js/cg-text-edit-pane.fasl")
(#x822344483cacb87b2f0cf64c24c6f38f . "./code.cg2js/cg-utility-dialog.fasl")
(#xd696385e125a62b4c5c48e30a664f19e . "./code.cg2js/cg-object-editor.fasl")
(#x403b5f8aa28f0be7a51a15d55849270f . "./code.cg2js/cg-edit-in-place.fasl")
(#x1e2807c6570943ddf47791a6b05b187a . "./code.cg2js/cg-radio-button.fasl")
(#xaebafe36c72d8075cf1a320af16319cb . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xdab7e06208626e7e4d99cfdb9d80b345 . "./code.cg2js/cg-nodes.fasl")
(#x8b7131cb9bab04bdb5f7b7534a914350 . "./code.cg2js/cg-get-position.fasl")
(#x02f06e23c0db3489409f5bb1bbf0d839 . "./code.cg2js/cg-directory-dialog.fasl")
(#x4c33d1d1b62af104409bee555c08acaa . "./code.cg2js/cg-lisp-text.fasl")
(#xe42de8a6e8db16227be3b87bf4e4e4ea . "./code.cg2js/cg-combo-box.fasl")
(#xfebbf70a6ba09dfb1396bd6844c29bc4 . "./code.cg2js/cg-text-widget.fasl")
(#x33070e4f4f3133cced4f4ccb740ed06d . "./code.cg2js/cg-list-view.fasl")
(#x6d1cb5ced856a634d74e5a8f0f097802 . "./code.cg2js/cg-graphics-context.fasl")
(#x5a68e83e4e9707fec9d689701c0976a9 . "./code.cg2js/cg-get-pixmap.fasl")
(#x0768885c3b31e66cfe924ea9599df0da . "./code.cg2js/cg-menu.fasl")
(#x6a08610fd0810756dd3388c8be08f27f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x2dc53a2ba8e51eb3db8a438ac702d0fa . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x30891d9d67cf012566e6dcfd0109ace0 . "./code.cg2js/cg-tooltip.fasl")
(#xfd5b880e868855550dd6f5fa32ffe0b6 . "./code.cg2js/cg-html-widget.fasl")
(#x6fce8ecf978c4e5db54234814c9135ad . "./code.cg2js/cg-choice-list.fasl")
(#xf9da03b709273d697939c98399855274 . "./code.cg2js/cg-icon.fasl")
(#x6d900a168e03c674d9ce6d16ee7e815b . "./code.cg2js/cg-acache.fasl")
(#xf20fd0ece8167a76443f9f96b5773b7b . "./code.cg2js/cg-template-string.fasl")
(#x7737d2badbb00729aac7a15f19dde663 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xf0cd9e2efd9591244e12835177bba879 . "./code.cg2js/cg-toggling-widget.fasl")
(#x435bc2dcc47667da182e88708dcf0110 . "./code.cg2js/cg-tab-control.fasl")
(#x7f258c0512421e733c19bc4f13ee1102 . "./code.cg2js/cg-split-bar.fasl")
(#x48f0438ae856c6e3d23fe9149085ab10 . "./code.cg2js/cg-picture-widget.fasl")
(#xbd3dfbcec3e20c7f4f0d2df7820abf3b . "./code.cg2js/cg-player.fasl")
(#x7606fabb3065af23fb81e9fbdb2f3c92 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xb835e45a3bf6eb990069b9c1e4cefe4e . "./code.cg2js/cg-choose-printer.fasl")
(#xf82465b156b88fd18dfa287b24b445d9 . "./code.cg2js/cg-os-window.fasl")
(#x225eaef891c8d4b0512c871f84815b0e . "./code.cg2js/cg-scroll-bar.fasl")
(#xd04b62c2da53978e6d2bea7760db5003 . "./code.cg2js/cg-clipboard.fasl")
(#x0a96438794fbb1f952b35129fa22bf1f . "./code.cg2js/cg-find-string-dialog.fasl")
(#x69007262793e449eff301a3b203bde4f . "./code.cg2js/cg-hotspot.fasl")
(#x3bd1c8fd20c20b076901d7270adf629a . "./code.cg2js/cg-check-box.fasl")
(#x1bc23e3224e2286b0493947dfaf7b044 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xee1f58e02732146fa961d407663de91f . "./code.cg2js/cg-file-dialog.fasl")
(#xd8a39fbba514a631288fd11808659be4 . "./code.cg2js/cg-toolbar.fasl")
(#x1aca78726c339f3f2d31d14070fdf000 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xc52089e972a696839d145154769e7300 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xf742976a974b2ef9d7e6797c1f3c3ecb . "./code.cg2js/cg-shortcut-menu.fasl")
(#x919248baae81fd0c9f729c6ceb7abaf4 . "./code.idejs/ide-project.fasl")
(#x8744b81b92ca3a2e09a290f0a46b098f . "./code.idejs/ide-intro.fasl")
(#x1fc58b0c3e4706e62a12b071f654810a . "./code.idejs/ide-find-in-files.fasl")
(#x5c04894df27e1e670e55f2f2d2642b10 . "./code.idejs/ide-grapher.fasl")
(#x67ac2dba3fd17be0bcfc8c7e34951a15 . "./code.idejs/ide-inspector.fasl")
(#xd05ff76912647ea75ef1a9536f223c8f . "./code.idejs/ide-editor.fasl")
(#x616fb29f9c71061b242112e56f70e66e . "./code.idejs/ide-window-list.fasl")
(#x5d11346b6c89806f4b550d124f5e7854 . "./code.idejs/ide-process-browser.fasl")
(#xac39bba5aff60caca17774d62155ca8e . "./code.idejs/ide-patch-dialog.fasl")
(#x36bda8b4fb12de2ba9ffc22200a49939 . "./code.idejs/ide-class-grapher.fasl")
(#x3bb0b2d2ebb58ba619b89f77424fd09b . "./code.idejs/ide-base.fasl")
(#x1dfb470d46fdb6c3271761f38a318925 . "./code.idejs/ide.fasl")
(#xa8468d9a61c1eecc7a39b3c4a72e8321 . "./code.idejs/ide-trace.fasl")
(#x7921803469e32f463d79b90549afd206 . "./code.idejs/ide-form.fasl")
(#x5e498de6db09c1123ed1dbaead9f817d . "./code.idejs/ide-find-definitions.fasl")
(#xdd6e7f2d1ed7e2bc5088085949028d06 . "./code.idejs/ide-apropos.fasl")
(#xc127d61af62fbae19ebdfb6d128c412d . "./code.idejs/ide-class-browser.fasl")
(#x67f8bbd1a18fd7a48bbcd7b8cedfdaaf . "./code.idejs/ide-navigator.fasl")
(#xa78d7605c90f05cd88f0b815461a1bd1 . "./code.idejs/ide-clipboard.fasl")
(#x9499012f5d1737e3abff344affd0d82b . "./code.idejs/ide-acache.fasl")
(#x03dc195a4324238d46eeb1ad9211204a . "./code.idejs/ide-keyshort.fasl")
(#xc9e03ae26894bd7ba18511e018c9a59e . "./code.idejs/ide-package-list.fasl")
(#x987dd152f65786744f66e5fe048ba294 . "./code.idejs/ide-dev.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
